package Monitor::eventlog;

require 5.001;
use Win32;
use Win32::EventLog;

use strict;
use vars qw(@ISA @EXPORT $VERSION);
use Monitor::logfile;

@ISA = qw(Monitor::logfile);

$Win32::EventLog::GetMessageText = 1;


sub stat {
    return();
}


sub defaults {
    my( $self ) = @_;

    $self->{"catchupsize"} = 200;
}



sub open {
    my( $self ) = @_;
    my $i;
    my( $file, $host );

    $host = $file = $self->{"file"};
    $host =~ s/:.*//;
    $file =~ s/^.*://;
    $self->{"open"} = $self->{"fh"} = Win32::EventLog->new( $file, "\\\\".$host );
    print "opening EventLog $file on host $host\n" if( $BigSister::common::dl>4 );
    unless( $self->{"open"} ) { return( 0 ); }
    $self->{"fh"}->GetNumber($i);
    my $oldest;
    $self->{"fh"}->GetOldest($oldest);
    $self->{"_nextevent"} = $oldest;
    $self->{"_max"} = $i+$oldest;
    return( $self->{"open"} );
}


sub seek {
    my( $self, $pos, $whence ) = @_;

    return(undef) unless( $self->{"open"} );
    if( $whence == 0 ) {
	$self->{"_nextevent"} = $pos;
    }
    elsif( $whence == 1 ) {
	$self->{"_nextevent"} += $pos;
    }
    else {
	my( $i, $oldest );
	$self->{"fh"}->GetNumber($i);
	$self->{"fh"}->GetOldest($oldest);
	$i = $i+$oldest+$pos;
	($i=1) if( $i<1 );
	$self->{"_nextevent"} = $i;
    }
    return 1;
}


sub tell {
    my( $self ) = @_;

    return $self->{"_nextevent"};
}


sub readline {
    my( $self ) = @_;
    my( $max, $event, $oldest );

    unless( $self->{"open"} ) {
	return(undef) unless( $self->open() );
	BigSister::common::log( "err", "established connection to EventLog ".($self->{"file"}) );
    }
    unless( $self->{"fh"}->GetNumber($max) && $self->{"fh"}->GetOldest($oldest) ) {
	BigSister::common::log( "warn", "EventLog GetNumber failed - trying re-open" );
	$self->open();
	unless( $self->{"fh"}->GetNumber($max) && $self->{"fh"}->GetNumber($oldest) ) {
	    BigSister::common::log( "err", "probably lost connection to EventLog ".($self->{"file"}) );
	    return undef;
	}
    }
    $max += $oldest;
    if( $max < $self->{"_max"} ) {
	$self->{"_nextevent"} = $max;
    }
    if( $self->{"_nextevent"} < $oldest ) {
        $self->{"_nextevent"} = $oldest;
    }
    if( $self->{"_nextevent"} > $max ) {
        $self->{"_nextevent"} = $max;
    }
    $self->{"_max"} = $max;
    return(undef) if( $self->{"_nextevent"} >= $max );
    $self->{"fh"}->Read( (EVENTLOG_SEEK_READ|EVENTLOG_FORWARDS_READ),$self->{"_nextevent"},$event);
    unless( $event->{"TimeGenerated"} ) {
        # reopen just in case ...
	my $next = $self->{"_nextevent"};
        $self->open();
	$self->{"_nextevent"} = $next;
    }
    $self->{"_nextevent"}++;
    $event->{"Message"} = $event->{"Message"} || $self->{"fh"}->GetMessageText( $event );
    return $event;
}



sub decode {
    my( $self, $line ) = @_;
    my $computer;

    $computer = $line->{"Computer"};
    $computer =~ tr/A-Z/a-z/;
    return( $line->{"TimeGenerated"}, 
	"EventID=".($line->{"EventID"})."; Source=".($line->{"Source"})."; Severity=".($line->{"EventType"}).": ".($line->{"Message"}), $computer );
}

1;
