# LDAP and LDAPS server monitor
#
# Gert Dewit <gert.dewit@sos.be>
#
# $Id: ldap.pm,v 1.1 2004/06/22 07:01:52 gertdewit Exp $
#
# Check the README file for more information, after installation it should be in
# the $BSHOME/moduleinfo/ directory

package Requester::ldap;

use strict;
use Requester::Requester;

@Requester::ldap::ISA = qw(Requester::Requester);

use MicroTime;
use Net::LDAP;

my $ldap = new Requester::ldap();

sub new
{
   my( $this, $args ) = @_;
   my $class = ref($this) || $this;
   my $self = {};
   bless( $self, $class );
   $self->register("ldap");
   return( $self );
}


sub getinfo {
    my( $self, $vars, $indexes ) = @_;

    return undef unless( $indexes );
    my $info = {};

    my $env = $self->{"env"};
    my $url = $env->expand_value( "url" );
    my $timeout = $env->expand_value( "timeout" );
    my $dn = $env->expand_value( "dn" );
    my $pass = $env->expand_value( "password" );
    my $base = $env->expand_value( "base" );
    my $filter = $env->expand_value( "filter" );
    my $version = $env->expand_value( "version" );

    foreach my $test (@$vars) {
        unless( $test =~ /^(time|status|message|log)\.(\d+)$/ ) {
	    print "   ldap cannot handle $test\n" if( $BigSister::common::dl > 4 );
	    next;
	}
	my $id = $2;
	next if( defined $info->{$id} );
	$info->{$id} = {};

	my $conn;
	$conn = Net::LDAP->new($url, timeout => $timeout);
	if ( not $conn ) {
	    $info->{$id}->{"time"} = -1;
	    $info->{$id}->{"message"} = "Connection refused";
	    $info->{$id}->{"log"} = "Connection refused";
	    $info->{$id}->{"status"} = 0;
	}
	else {
	    my $msg;
	    my $start = MicroTime::gettimeofday();
	    if( $dn ) {
		$msg = $conn->bind($dn, password=> $pass, version => $version);
	    }
	    else {
		$msg = $conn->bind(version => $version);
	    }
	    if ( $msg->code ) {
		$info->{$id}->{"time"} = -1;
		$info->{$id}->{"message"} = $msg->error;
		$info->{$id}->{"log"} = $msg->error;
		$info->{$id}->{"status"} = 0;
	    }
	    else {
		if( $filter ) {
		    $msg = $conn->search(base => $base, filter => $filter, attrs => ['dn']);
	    	    my $responsetime = int( MicroTime::difftime( $start, MicroTime::gettimeofday() ) * 100 + 0.5 ) / 100;
		    if( $msg->code ) {
			$info->{$id}->{"time"} = $responsetime;
			$info->{$id}->{"message"} = $msg->error;
			$info->{$id}->{"log"} = $msg->error." for base=".$base." filter=".$filter;
			$info->{$id}->{"status"} = 1;
		    }
		    else {
			$info->{$id}->{"time"} = $responsetime;
			$info->{$id}->{"message"} = "LDAP search OK";
			$info->{$id}->{"log"} = $msg->count." entries returned for base=".$base." filter=".$filter;
			$info->{$id}->{"status"} = 3;
		    }
		}
		else {
	    	    my $responsetime = int( MicroTime::difftime( $start, MicroTime::gettimeofday() ) * 100 + 0.5 ) / 100;
		    $info->{$id}->{"time"} = $responsetime;
		    $info->{$id}->{"message"} = "LDAP bind OK";
		    $info->{$id}->{"log"} = "LDAP bind OK";
		    $info->{$id}->{"status"} = 3;
		}
		$conn->unbind;
	    }
        }
    }
    return $info;
}

1;
