# Mail Queue monitor
#
# Gert Dewit <gert.dewit@sos.be>
#
# $Id : $
#
# Check the README file for more information, after installation it should be in
# the $BSHOME/moduleinfo/ directory

package Requester::mailq;

use strict;

@Requester::mailq::ISA = qw(Requester::Requester);

my $mailq = new Requester::mailq();

sub new
{
    my ( $this, $args ) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};
    bless( $self, $class );
    $self->register("mailq");
    return( $self );
}

sub getinfo {
    my( $self, $vars, $indexes ) = @_;

    return undef unless( $indexes );
    my $info = {};

    my $env = $self->{"env"};
    my $mqcmd = $env->expand_value( "mqcmd" );
    my $warn = $env->expand_value( "warn" );
    my $fail = $env->expand_value( "fail" );
    $mqcmd = "/usr/bin/mailq" unless( $mqcmd );
    $warn = 500 unless( $warn );
    $fail = 750 unless( $fail );

    foreach my $test (@$vars) {
	unless( $test =~ /^(messages|messagesNow|addresses|addressesNow|status|message)\.(\d+)$/ ) {
	   next;
	}
	my $id = $2;
	next if( defined $info->{$id} );
	$info->{$id} = {};
	if (open(QUEUE, "$mqcmd|")) {
	    my $line;
	    my ($messages, $addresses, $messagesNow, $addressesNow) = (0, 0, 0, 0);
	    my $now;
		
	    while ($line = <QUEUE> && $line !~ /^[A-Za-z0-9]/) {
		next;   # Skip the header
	    }

	    while ($line = <QUEUE>) {
		if ($line =~ /^[A-Za-z0-9]+/) {
		    next if( $line =~ /Queue status\./ );
		    # ID* means it is being worked on, ID means it is waiting
		    if ($line =~ /^\S+\*/) {
			$now = 1;
			$messagesNow++;
		    } else {
			$now = 0;
			$messages++;
		    }
		} elsif ($line =~ /^\s/) {
		    next if ($line =~ /^\s+\(/); # An error message
		    next if ($line =~ /^\s+$/); # Empty line
		    # An address
		    next if( $line =~ /Total requests:/ );
		    if ($now) {
			$addressesNow++;
		    }
		    else {
			$addresses++;
		    }
		}
	    }
	    if(close(QUEUE)) {
		$info->{$id}->{"status"} = 3;
		$info->{$id}->{"message"} = "'$mqcmd' ran OK";
	    }
	    else {
		$info->{$id}->{"status"} = 1;
		$info->{$id}->{"message"} = "Could not close '$mqcmd'";
	    }
	    if($messages > $fail) {
		$info->{$id}->{"status"} = 0;
		$info->{$id}->{"message"} = "$messages messages in the queue (more than $fail)";
	    }
	    elsif($messages > $warn) {
		$info->{$id}->{"status"} = 1;
		$info->{$id}->{"message"} = "$messages messages in the queue (more than $warn)";
	    }
	    $info->{$id}->{"addresses"} = $addresses;
	    $info->{$id}->{"addressesNow"} = $addressesNow;
	    $info->{$id}->{"messages"} = $messages;
	    $info->{$id}->{"messagesNow"} = $messagesNow;
	}
	else {
		$info->{$id}->{"addresses"} = -1;
		$info->{$id}->{"addressesNow"} = -1;
		$info->{$id}->{"messages"} = -1;
		$info->{$id}->{"messagesNow"} = -1;
		$info->{$id}->{"message"} = "Could not run '$mqcmd'";
		$info->{$id}->{"status"} = 0;
	}
    }
    return $info;
}


1;
